@@ -54,11 +54,15 @@ static uint32_t RPC_getHashVal(uint8_t *key,uint32_t keyLength,uint32_t tableSiz
 
 static RpcBool_t RPC_checkTimeoutInternal(struct timeval *timeout,struct timeval *curTime);
 
+#ifdef	USE_CANCEL_TASK
+static void RPC_taskCntlCleanup(void *arg);
+#endif	/* USE_CANCEL_TASK */
+
 /* ----- RPC-LIB APIs ----- */
 
 /*--- TASK CONTROL LIBRARY ---*/
 
-RpcResult_t RPC_CreateTaskControl(RpcBool_t IpcFlag,RpcTaskControl_t *tskCntl) {
+extern RpcResult_t RPC_CreateTaskControl(RpcBool_t IpcFlag,RpcTaskControl_t *tskCntl) {
 	RpcResult_t ret;
 
 	if(tskCntl == (RpcTaskControl_t*)NULL)
@@ -102,7 +106,7 @@ RpcResult_t RPC_CreateTaskControl(RpcBool_t IpcFlag,RpcTaskControl_t *tskCntl) {
 	return RPC_SUCCESS;
 }
 
-void RPC_DestroyTaskControl(RpcTaskControl_t *tskCntl) {
+extern void RPC_DestroyTaskControl(RpcTaskControl_t *tskCntl) {
 	if(tskCntl == (RpcTaskControl_t*)NULL)
 		return;
 #ifndef	USE_SEMAPHORE
@@ -113,7 +117,7 @@ void RPC_DestroyTaskControl(RpcTaskControl_t *tskCntl) {
 #endif	/* USE_SEMAPHORE */
 }
 
-RpcResult_t RPC_TaskControlWait(RpcTaskControl_t *tskCntl) {
+extern RpcResult_t RPC_TaskControlWait(RpcTaskControl_t *tskCntl) {
 	RpcResult_t ret;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_DEBUG2,"IN RPC_TaskControlWait");
@@ -124,11 +128,17 @@ RpcResult_t RPC_TaskControlWait(RpcTaskControl_t *tskCntl) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_MutexLock ret:%s",RPC_LogConvertResultCode(ret));
 		return ret;			
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPushCleanup(RPC_taskCntlCleanup,(void*)tskCntl);
+#endif	/* USE_CANCEL_TASK */
 	if((ret = RPC_CondWait(&(tskCntl->condId),&(tskCntl->mutexId))) != RPC_SUCCESS) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_CondWait ret:%s",RPC_LogConvertResultCode(ret));
 		(void)RPC_MutexUnlock(&(tskCntl->mutexId));
 		return ret;			
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPopCleanup(0);
+#endif	/* USE_CANCEL_TASK */
 	(void)RPC_MutexUnlock(&(tskCntl->mutexId));
 #else	/* USE_SEMAPHORE */
 	if((ret = RPC_SemWait(&(tskCntl->semId))) != RPC_SUCCESS) {
@@ -140,7 +150,7 @@ RpcResult_t RPC_TaskControlWait(RpcTaskControl_t *tskCntl) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskControlTimedWait(RpcTaskControl_t *tskCntl, struct timespec *specTime) {
+extern RpcResult_t RPC_TaskControlTimedWait(RpcTaskControl_t *tskCntl, struct timespec *specTime) {
 	RpcResult_t ret;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_DEBUG2,"IN RPC_TaskControlTimedWait");
@@ -151,6 +161,9 @@ RpcResult_t RPC_TaskControlTimedWait(RpcTaskControl_t *tskCntl, struct timespec
 		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_MutexLock ret:%s",RPC_LogConvertResultCode(ret));
 		return ret;			
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPushCleanup(RPC_taskCntlCleanup,(void*)tskCntl);
+#endif	/* USE_CANCEL_TASK */
 	if((ret = RPC_CondWaitTimeout(&(tskCntl->condId),&(tskCntl->mutexId),specTime)) != RPC_SUCCESS) {
 		if(ret == RPC_TIMEOUT_ERROR)
 			(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_WARNING, " TIMEOUT in RPC_CondWaitTimeout ret:%s",RPC_LogConvertResultCode(ret));
@@ -159,13 +172,16 @@ RpcResult_t RPC_TaskControlTimedWait(RpcTaskControl_t *tskCntl, struct timespec
 		(void)RPC_MutexUnlock(&(tskCntl->mutexId));
 		return ret;
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPopCleanup(0);
+#endif	/* USE_CANCEL_TASK */
 	(void)RPC_MutexUnlock(&(tskCntl->mutexId));
 #else	/* USE_SEMAPHORE */
 	if((ret = RPC_SemTimedWait(&(tskCntl->semId),specTime)) != RPC_SUCCESS) {
 		if(ret == RPC_TIMEOUT_ERROR)
-			(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_WARNING, " TIMEOUT in RPC_CondWaitTimeout ret:%s",RPC_LogConvertResultCode(ret));
+			(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_WARNING, " TIMEOUT in RPC_SemTimedWait ret:%s",RPC_LogConvertResultCode(ret));
 		else
-			(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_CondWaitTimeout ret:%s",RPC_LogConvertResultCode(ret));
+			(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_SemTimedWait ret:%s",RPC_LogConvertResultCode(ret));
 		return ret;
 	}
 #endif	/* USE_SEMAPHORE*/
@@ -173,13 +189,14 @@ RpcResult_t RPC_TaskControlTimedWait(RpcTaskControl_t *tskCntl, struct timespec
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskControlSignal(RpcTaskControl_t *tskCntl) {
+extern RpcResult_t RPC_TaskControlSignal(RpcTaskControl_t *tskCntl) {
 	RpcResult_t ret;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_DEBUG2,"IN RPC_TaskControlSignal");
 	if(tskCntl == (RpcTaskControl_t*)NULL)
 		return RPC_PARAM_ERROR;
 #ifndef	USE_SEMAPHORE
+#if	0
 	if((ret = RPC_MutexLock(&(tskCntl->mutexId))) != RPC_SUCCESS) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_MutexLock ret:%s",RPC_LogConvertResultCode(ret));
 		return ret;
@@ -190,6 +207,12 @@ RpcResult_t RPC_TaskControlSignal(RpcTaskControl_t *tskCntl) {
 		return ret;
 	}
 	(void)RPC_MutexUnlock(&(tskCntl->mutexId));
+#else	/* 0 */
+	if((ret = RPC_CondSignal(&(tskCntl->condId))) != RPC_SUCCESS) {
+		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_CondSignal ret:%s",RPC_LogConvertResultCode(ret));
+		return ret;
+	}
+#endif	/* 0 */
 #else	/* USE_SEMAPHORE */
 	if((ret = RPC_SemPost(&(tskCntl->semId))) != RPC_SUCCESS) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_FATAL, " ERROR in RPC_SemPost ret:%s",RPC_LogConvertResultCode(ret));
@@ -200,7 +223,7 @@ RpcResult_t RPC_TaskControlSignal(RpcTaskControl_t *tskCntl) {
 	return RPC_SUCCESS;
 }
 
-void IPC_ResetTaskControl(RpcTaskControl_t *tskCntl) {
+extern void IPC_ResetTaskControl(RpcTaskControl_t *tskCntl) {
 #ifndef	USE_SEMAPHORE
 #else	/* USE_SEMAPHORE */
 	if(tskCntl == (RpcTaskControl_t*)NULL)
@@ -219,11 +242,11 @@ void IPC_ResetTaskControl(RpcTaskControl_t *tskCntl) {
 }
 /*--- HASH LIBRARY ---*/
 
-void RPC_RegisterHashTable(uint32_t *hashRandTable) {
+extern void RPC_RegisterHashTable(uint32_t *hashRandTable) {
 	RpcHashRandTable = hashRandTable;
 }
 
-RpcResult_t RPC_AddHashEntry(RpcHashEntry_t *hashTblTop,void *entry,uint8_t *val, uint32_t valSize,uint32_t tblSize) {
+extern RpcResult_t RPC_AddHashEntry(RpcHashEntry_t *hashTblTop,void *entry,uint8_t *val, uint32_t valSize,uint32_t tblSize) {
 	uint32_t hashId;RpcHashEntry_t *hashTbl;int i;
 
 	if((hashTblTop == (RpcHashEntry_t*)NULL)||(entry == (void*)NULL)||(val == (uint8_t*)NULL)||(valSize == 0))
@@ -242,7 +265,7 @@ RpcResult_t RPC_AddHashEntry(RpcHashEntry_t *hashTblTop,void *entry,uint8_t *val
 	return RPC_NO_MORE_RESOURCE;
 }
 
-RpcResult_t RPC_DeleteHashEntry(RpcHashEntry_t *hashTblTop,uint8_t *val, uint32_t valSize,uint32_t tblSize,RPC_CompareHashVal_t compareHashVal) {
+extern RpcResult_t RPC_DeleteHashEntry(RpcHashEntry_t *hashTblTop,uint8_t *val, uint32_t valSize,uint32_t tblSize,RPC_CompareHashVal_t compareHashVal) {
 	uint32_t hashId,nextId;RpcHashEntry_t *hashTbl,*nextTbl;int i;
 
 	if((hashTblTop == (RpcHashEntry_t*)NULL)||(val == (uint8_t*)NULL)||(valSize == 0)||(compareHashVal == (RPC_CompareHashVal_t)NULL))
@@ -271,7 +294,7 @@ RpcResult_t RPC_DeleteHashEntry(RpcHashEntry_t *hashTblTop,uint8_t *val, uint32_
 	return RPC_PARAM_ERROR;
 }
 
-void *RPC_SearchEntryByHash(RpcHashEntry_t *hashTblTop, uint8_t *val, uint32_t valSize, uint32_t tblSize, RPC_CompareHashVal_t compareHashVal) {
+extern void *RPC_SearchEntryByHash(RpcHashEntry_t *hashTblTop, uint8_t *val, uint32_t valSize, uint32_t tblSize, RPC_CompareHashVal_t compareHashVal) {
 	uint32_t hashId;RpcHashEntry_t *hashTbl;int i;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_LIB,RPC_LOG_LEVEL_DEBUG2,"IN RPC_SearchEntryByHash PID:0x%x; hashTop:0x%x; val:0x%x;(0x%x) valSize:%d; tblSize:%d;",
@@ -303,7 +326,7 @@ void *RPC_SearchEntryByHash(RpcHashEntry_t *hashTblTop, uint8_t *val, uint32_t v
 
 /*--- LIST LIBRARY --- */
 
-void *RPC_AllocListData(RpcList_t **listTop,uint32_t entitySize,uint32_t allocId,RPC_AllocFunc_t allocFunc,RPC_FreeFunc_t freeFunc) {
+extern void *RPC_AllocListData(RpcList_t **listTop,uint32_t entitySize,uint32_t allocId,RPC_AllocFunc_t allocFunc,RPC_FreeFunc_t freeFunc) {
 	RpcList_t *list;
 
 	if(listTop == (RpcList_t**)NULL)
@@ -320,7 +343,7 @@ void *RPC_AllocListData(RpcList_t **listTop,uint32_t entitySize,uint32_t allocId
 	return list->entry;
 }
 
-void RPC_FreeListData(RpcList_t **listTop,void *delEntry,uint32_t allocId,RPC_FreeFunc_t freeFunc) {
+extern void RPC_FreeListData(RpcList_t **listTop,void *delEntry,uint32_t allocId,RPC_FreeFunc_t freeFunc) {
 	RpcList_t *list,*prev;
 
 	if((listTop == (RpcList_t**)NULL)||(delEntry == (void*)NULL))
@@ -342,7 +365,28 @@ void RPC_FreeListData(RpcList_t **listTop,void *delEntry,uint32_t allocId,RPC_Fr
 	}
 }
 
-void RPC_SearchAndFreeListData(RpcList_t **listTop,uint32_t id,uint32_t allocId,RPC_CompareFunc compareFunc,RPC_FreeFunc_t freeFunc) {
+extern RpcList_t *RPC_RemoveListEntry(RpcList_t **listTop,void *remEntry) {
+	RpcList_t *list,*prev;
+
+	if((listTop == (RpcList_t**)NULL)||(remEntry == (void*)NULL))
+		return;
+	prev = (RpcList_t*)NULL;
+	for(list = *listTop;list != (RpcList_t*)NULL;list = (RpcList_t*)list->next) {
+		if(remEntry == list->entry) {
+			if(prev == (RpcList_t*)NULL) {
+				(*listTop) = list->next;
+			}
+			else {
+				prev->next = list->next;
+			}
+			return list;
+		}
+		prev = list;
+	}
+	return (RpcList_t*)NULL;
+}
+
+extern void RPC_SearchAndFreeListData(RpcList_t **listTop,uint32_t id,uint32_t allocId,RPC_CompareFunc compareFunc,RPC_FreeFunc_t freeFunc) {
 	RpcList_t *list,*prev;
 
 	if(listTop == (RpcList_t**)NULL)
@@ -365,7 +409,7 @@ void RPC_SearchAndFreeListData(RpcList_t **listTop,uint32_t id,uint32_t allocId,
 	return;
 }
 
-void *RPC_SearchListEntryById(RpcList_t *listTop,uint32_t id,RPC_CompareFunc compareFunc) {
+extern void *RPC_SearchListEntryById(RpcList_t *listTop,uint32_t id,RPC_CompareFunc compareFunc) {
 	RpcList_t *list;
 
 	for(list = listTop;list != (RpcList_t*)NULL;list = (RpcList_t*)list->next) {
@@ -376,7 +420,7 @@ void *RPC_SearchListEntryById(RpcList_t *listTop,uint32_t id,RPC_CompareFunc com
 	return (void*)NULL;
 }
 
-void *RPC_SearchListEntryByIds(RpcList_t *listTop,uint32_t id1,uint32_t id2,RPC_compareFuncByIds compareFunc) {
+extern void *RPC_SearchListEntryByIds(RpcList_t *listTop,uint32_t id1,uint32_t id2,RPC_compareFuncByIds compareFunc) {
 	RpcList_t *list;
 
 	for(list = listTop;list != (RpcList_t*)NULL;list = (RpcList_t*)list->next) {
@@ -387,7 +431,7 @@ void *RPC_SearchListEntryByIds(RpcList_t *listTop,uint32_t id1,uint32_t id2,RPC_
 	return (void*)NULL;
 }
 
-uint32_t RPC_CountNumOfListEntry(RpcList_t *listTop) {
+extern uint32_t RPC_CountNumOfListEntry(RpcList_t *listTop) {
 	RpcList_t *list;uint32_t num = 0;
 
 	for(list = listTop;list != (RpcList_t*)NULL;list = (RpcList_t*)list->next) {
@@ -396,25 +440,25 @@ uint32_t RPC_CountNumOfListEntry(RpcList_t *listTop) {
 	return num;
 }
 
-void *RPC_ShmAllocMemWrapper(uint32_t allocId,uint32_t size) {
+extern void *RPC_ShmAllocMemWrapper(uint32_t allocId,uint32_t size) {
 	return IPC_AllocShmMem(allocId, size);
 }
 
-void RPC_ShmFreeMemWrapper(uint32_t allocId,void *area) {
+extern void RPC_ShmFreeMemWrapper(uint32_t allocId,void *area) {
 	IPC_FreeShmMem(allocId,area);
 }
 
-void *RPC_LclAllocMemWrapper(uint32_t allocId,uint32_t size) {
+extern void *RPC_LclAllocMemWrapper(uint32_t allocId,uint32_t size) {
 	return RPC_AllocMem(size);
 }
 
-void RPC_LclFreeMemWrapper(uint32_t allocId,void *area) {
+extern void RPC_LclFreeMemWrapper(uint32_t allocId,void *area) {
 	RPC_FreeMem(area);
 }
 
 /* --- TIME LIBRARY --- */
 
-RpcBool_t RPC_SetTimeout(uint32_t timeout, struct timeval *absTime) {
+extern RpcBool_t RPC_SetTimeout(uint32_t timeout, struct timeval *absTime) {
 	struct timeval curTime;uint32_t errCode;
 
 	if(absTime == (struct timeval*)NULL)
@@ -430,7 +474,7 @@ RpcBool_t RPC_SetTimeout(uint32_t timeout, struct timeval *absTime) {
 	return RPC_TRUE;
 }
 
-RpcBool_t RPC_CheckTimeout(struct timeval *timeout) {
+extern RpcBool_t RPC_CheckTimeout(struct timeval *timeout) {
 	struct timeval curTime;
 
 	if(timeout == (struct timeval*)NULL)
@@ -438,7 +482,7 @@ RpcBool_t RPC_CheckTimeout(struct timeval *timeout) {
 	return RPC_checkTimeoutInternal(timeout,&curTime);
 }
 
-RpcBool_t RPC_GetRestTimeout(struct timeval *absTime,uint32_t *restTimeout) {
+extern RpcBool_t RPC_GetRestTimeout(struct timeval *absTime,uint32_t *restTimeout) {
 	struct timeval curTime;
 
 	if((absTime == (struct timeval*)NULL)||(restTimeout == (uint32_t*)NULL))
@@ -449,7 +493,7 @@ RpcBool_t RPC_GetRestTimeout(struct timeval *absTime,uint32_t *restTimeout) {
 	return RPC_TRUE;
 }
 
-void RPC_ConvertTime(struct timeval *timeVal,struct timespec *timeSpec) {
+extern void RPC_ConvertTime(struct timeval *timeVal,struct timespec *timeSpec) {
 	if((timeVal == (struct timeval*)NULL)||(timeSpec == (struct timespec*)NULL))
 		return;
 	timeSpec->tv_sec = timeVal->tv_sec;
@@ -458,13 +502,20 @@ void RPC_ConvertTime(struct timeval *timeVal,struct timespec *timeSpec) {
 	timeSpec->tv_nsec = ((timeSpec->tv_nsec)%1000000000);
 }
 
-void RPC_ConvertTimeoutToTimeval(uint32_t timeout,struct timeval *timeVal) {
+extern void RPC_ConvertTimeoutToTimeval(uint32_t timeout,struct timeval *timeVal) {
 	if(timeVal == (struct timeval*)NULL)
 		return;
 	timeVal->tv_sec = timeout /1000;
 	timeVal->tv_usec = (timeout%1000)*1000000;
 }
 
+extern void RPC_ConvertTimevalToTimeout(struct timeval *timeVal,uint32_t *timeout) {
+	if((timeVal == (struct timeval*)NULL)||(timeout == (uint32_t*)NULL))
+		return;
+	*timeout = timeVal->tv_sec * 1000;
+	*timeout += (timeVal->tv_usec/1000);
+}
+
 /* ----- Internal Functions ----- */
 
 static uint32_t RPC_nextHashEntry(uint32_t hashId,uint32_t tblSize) {
@@ -498,10 +549,21 @@ static RpcBool_t RPC_checkTimeoutInternal(struct timeval *timeout,struct timeval
 		return RPC_FALSE;
 	else if(curTime->tv_sec < timeout->tv_sec)
 		return RPC_TRUE;
-	else if(curTime->tv_usec > timeout->tv_usec)
+	else if(curTime->tv_usec >= timeout->tv_usec)
 		return RPC_FALSE;
 	else
 		return RPC_TRUE;
 }
 
+#ifdef	USE_CANCEL_TASK
+static void RPC_taskCntlCleanup(void *arg) {
+#ifndef	USE_SEMAPHORE
+	RpcTaskControl_t *tskCntl;
+
+	tskCntl = (RpcTaskControl_t*)arg;
+	(void)RPC_MutexUnlock(&(tskCntl->mutexId));
+#endif	/* USE_SEMAPHORE */
+}
+#endif	/* USE_CANCEL_TASK */
+
 /* ----- TEST Functions ----- */
