@@ -28,7 +28,7 @@
  *
  */
 
-#define	IPC_PACK_MAIN
+#define	IPC_MARSHAL_MAIN
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -56,7 +56,7 @@ uint8_t *IPC_ConvertTypeToString(RpcParamType_t type);
 
 /*--- MARSHAL LIBRARY ---*/
 
-RpcResult_t IPC_InitMarshal(RpcBool_t masterProcess) {
+extern RpcResult_t IPC_InitMarshal(RpcBool_t masterProcess) {
 	RpcResult_t ret = RPC_SUCCESS;
 
 	if(IpcMarshalDataSharedMemId == -1) {
@@ -78,7 +78,7 @@ RpcResult_t IPC_InitMarshal(RpcBool_t masterProcess) {
 	return ret;
 }
 
-RpcResult_t IPC_Marshal(RpcParamData_t *inData, void **outData, uint32_t *outSize) {
+extern RpcResult_t IPC_Marshal(RpcParamData_t *inData, void **outData, uint32_t *outSize) {
 	RpcParamData_t *p;uint32_t useSize;RpcResult_t ret;
 	uint32_t oneSize,tmpSize;void *outP;RpcParamHeader_t *header;
 
@@ -128,7 +128,7 @@ RpcResult_t IPC_Marshal(RpcParamData_t *inData, void **outData, uint32_t *outSiz
 	return RPC_SUCCESS;
 }
 
-RpcResult_t IPC_Unmarshal(void *inData, uint32_t inSize, void *outData, uint32_t outSize) {
+extern RpcResult_t IPC_Unmarshal(void *inData, uint32_t inSize, void *outData, uint32_t outSize) {
 	RpcParamData_t *pData;void *inP,*outP;int32_t size,needSize,oSize;
 	RpcResult_t ret;RpcParamHeader_t *header;
 
@@ -188,7 +188,7 @@ RpcResult_t IPC_Unmarshal(void *inData, uint32_t inSize, void *outData, uint32_t
 	return RPC_SUCCESS;
 }
 
-void IPC_FreeMarshaledSharedData(void *data) {
+extern void IPC_FreeMarshaledSharedData(void *data) {
 	if(data == (void*)NULL)
 		return;
 	if(IpcMarshalDataSharedMemId != -1) {
@@ -196,7 +196,7 @@ void IPC_FreeMarshaledSharedData(void *data) {
 	}
 }
 
-uint32_t IPC_GetMarshaledDataSize(void *data) {
+extern uint32_t IPC_GetMarshaledDataSize(void *data) {
 	uint32_t size;RpcParamHeader_t *header;
 
 	if(data == (void*)NULL)
@@ -206,7 +206,7 @@ uint32_t IPC_GetMarshaledDataSize(void *data) {
 	return size;
 }
 
-void *IPC_DuplicateMarshaledData(void *marshaledData,uint32_t marshaledDataSize) {
+extern void *IPC_DuplicateMarshaledData(void *marshaledData,uint32_t marshaledDataSize) {
 	void *ret;
 
 	if((marshaledData == (void*)NULL)||(marshaledDataSize == (uint32_t)0))
@@ -220,7 +220,7 @@ void *IPC_DuplicateMarshaledData(void *marshaledData,uint32_t marshaledDataSize)
 
 /* --- OTHER LIBRARY --- */
 
-void IPC_SetMsgParam(RpcMsgParam_t *msgParam,uint32_t funcId,uint32_t pId,uint32_t queId,uint32_t seqNum,uint32_t extData,void* extParam) {
+extern void IPC_SetMsgParam(RpcMsgParam_t *msgParam,uint32_t funcId,uint32_t pId,uint32_t queId,uint32_t seqNum,uint32_t extData,void* extParam) {
 	msgParam->u.msgParam.funcId = funcId;
 	msgParam->u.msgParam.srcPid = pId;
 	msgParam->u.msgParam.srcQueId = queId;
