@@ -104,12 +104,15 @@ static IpcQueControl_t *IPC_getQueCtrl(RpcBool_t IpcFlag);
 
 static RpcBool_t IPC_compareQueInternalByQueId(RpcList_t *list,uint32_t queId);
 
+#ifdef	USE_CANCEL_TASK
+static void IPC_msgQueCleanup(void *arg);
+#endif	/* USE_CANCEL_TASK */
+
 #ifdef	IPC_USE_HASH_FUNC_FOR_MSG_QUE
 static RpcHashEntry_t *IPC_initMsgQueHash(RpcBool_t IpcFlag,uint32_t hashTableSize);
 static RpcBool_t IPC_compareQueIdbyHash(void *entry, uint8_t *val, uint32_t valSize);
 #endif	/* IPC_USE_HASH_FUNC_FOR_MSG_QUE */
 
-
 static void IPC_printListData(RpcBool_t IpcFlag,IpcQueControl_t *mqcp );
 #ifdef	IPC_USE_HASH_FUNC_FOR_MSG_QUE
 static void IPC_printQueHash(RpcBool_t IpcFlag, IpcQueControl_t *mqcp);
@@ -117,18 +120,18 @@ static void IPC_printQueHash(RpcBool_t IpcFlag, IpcQueControl_t *mqcp);
 
 /* ----- IPC-QUEUE APIs ----- */
 
-RpcResult_t IPC_InitQue(RpcBool_t masterProcess,RpcBool_t IpcFlag) {
+extern RpcResult_t IPC_InitQue(RpcBool_t masterProcess,RpcBool_t IpcFlag) {
 	if(IpcFlag)
 		return IPC_InitExtQue(masterProcess);
 	else
 		return IPC_initQueInternal(RPC_FALSE);
 }
 
-RpcResult_t IPC_CreateQue(RpcBool_t IpcFlag,uint32_t queSize, uint32_t *queId) {
+extern RpcResult_t IPC_CreateQue(RpcBool_t IpcFlag,uint32_t queSize, uint32_t *queId) {
 	return IPC_createQueInternal(IpcFlag,queSize,queId);
 }
 
-RpcResult_t IPC_DestroyQue(uint32_t queId) {
+extern RpcResult_t IPC_DestroyQue(uint32_t queId) {
 	RpcResult_t ret;
 
 	if((ret = IPC_destroyQueInternal(RPC_FALSE, queId)) == RPC_SUCCESS) {
@@ -142,7 +145,7 @@ RpcResult_t IPC_DestroyQue(uint32_t queId) {
 	}
 }
 
-RpcResult_t IPC_EnableQue(uint32_t queId) {
+extern RpcResult_t IPC_EnableQue(uint32_t queId) {
 	RpcResult_t ret;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_MSG_QUE,RPC_LOG_LEVEL_DEBUG1,"IN IPC_EnableQue pid:0x%x; queId:0x%x;", RPC_GetPid(), queId);
@@ -157,7 +160,7 @@ RpcResult_t IPC_EnableQue(uint32_t queId) {
 	}
 }
 
-RpcResult_t IPC_DisableQue(uint32_t queId) {
+extern RpcResult_t IPC_DisableQue(uint32_t queId) {
 	RpcResult_t ret;
 
 	if((ret = IPC_disableQueInternal(RPC_FALSE, queId)) == RPC_SUCCESS) {
@@ -171,7 +174,7 @@ RpcResult_t IPC_DisableQue(uint32_t queId) {
 	}
 }
 
-RpcResult_t IPC_GetMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_GetMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_MSG_QUE,RPC_LOG_LEVEL_DEBUG1, "IN IPC_GetMsgFromQue queId:0x%x; msgId:0x%x; msgParam:0x%x, timeout:%d;",queId,msgId,msgParam,timeout);
@@ -191,7 +194,7 @@ RpcResult_t IPC_GetMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *ms
 	}
 }
 
-RpcResult_t IPC_SweepMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam) {
+extern RpcResult_t IPC_SweepMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam) {
 	IpcQueManage_t *queManage;
 
 	if((msgId == (uint32_t*)NULL)||(msgParam == (RpcMsgParam_t*)NULL)) {
@@ -209,7 +212,7 @@ RpcResult_t IPC_SweepMsgFromQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *
 	}
 }
 
-RpcResult_t IPC_PutMsgToQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_PutMsgToQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 	if((queManage = IPC_getQueManage(RPC_FALSE, queId)) != (IpcQueManage_t*)NULL) {
@@ -223,7 +226,7 @@ RpcResult_t IPC_PutMsgToQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgPa
 	}
 }
 
-RpcResult_t IPC_PutMsgToQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_PutMsgToQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 	if((queManage = IPC_getQueManage(RPC_FALSE,  queId)) != (IpcQueManage_t*)NULL) {
@@ -237,7 +240,7 @@ RpcResult_t IPC_PutMsgToQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam_t
 	}
 }
 
-RpcResult_t IPC_GetNumOfMsg(uint32_t queId, uint32_t *numOfMsg) {
+extern RpcResult_t IPC_GetNumOfMsg(uint32_t queId, uint32_t *numOfMsg) {
 	IpcQueManage_t *queManage;
 
 	if(numOfMsg == (uint32_t*)NULL)
@@ -255,7 +258,7 @@ RpcResult_t IPC_GetNumOfMsg(uint32_t queId, uint32_t *numOfMsg) {
 }
 
 #ifdef	USE_RENDEZVOUS
-IpcRendezvControl_t *IPC_GetRendezvControl(void) {
+extern IpcRendezvControl_t *IPC_GetRendezvControl(void) {
 	void *startAddr;uint32_t gapSize;IpcRendezvControl_t *rcp;
 
 	if(IpcShmMsgQueFd == -1)
@@ -272,7 +275,7 @@ IpcRendezvControl_t *IPC_GetRendezvControl(void) {
 
 /*--- SHM MSG QUE APIs ---*/
 
-RpcResult_t IPC_InitExtQue(RpcBool_t masterProcess) {
+extern RpcResult_t IPC_InitExtQue(RpcBool_t masterProcess) {
 	uint32_t fd; void *addr; uint32_t size;RpcResult_t ret;
 	void *startArea;uint32_t gapSize;
 
@@ -317,23 +320,23 @@ RpcResult_t IPC_InitExtQue(RpcBool_t masterProcess) {
 	return ret;
 }
 
-RpcResult_t IPC_CreateExtQue(uint32_t queSize, uint32_t *queId) {
+extern RpcResult_t IPC_CreateExtQue(uint32_t queSize, uint32_t *queId) {
 	return IPC_createQueInternal(RPC_TRUE,queSize,queId);
 }
 
-RpcResult_t IPC_DestroyExtQue(uint32_t queId) {
+extern RpcResult_t IPC_DestroyExtQue(uint32_t queId) {
 	return IPC_destroyQueInternal(RPC_TRUE, queId);
 }
 
-RpcResult_t IPC_EnableExtQue( uint32_t queId) {
+extern RpcResult_t IPC_EnableExtQue( uint32_t queId) {
 	return IPC_enableQueInternal(RPC_TRUE, queId);
 }
 
-RpcResult_t IPC_DisableExtQue(uint32_t queId) {
+extern RpcResult_t IPC_DisableExtQue(uint32_t queId) {
 	return IPC_disableQueInternal(RPC_TRUE, queId);
 }
 
-RpcResult_t IPC_GetMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_GetMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 	if((msgId == (uint32_t*)NULL)||(msgParam == (RpcMsgParam_t*)NULL)) {
@@ -348,7 +351,7 @@ RpcResult_t IPC_GetMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t
 	}
 }
 
-RpcResult_t IPC_SweepMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam) {
+extern RpcResult_t IPC_SweepMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_t *msgParam) {
 	IpcQueManage_t *queManage;
 
 	if((msgId == (uint32_t*)NULL)||(msgParam == (RpcMsgParam_t*)NULL)) {
@@ -363,7 +366,7 @@ RpcResult_t IPC_SweepMsgFromExtQue(uint32_t queId, uint32_t *msgId, RpcMsgParam_
 	}
 }
 
-RpcResult_t IPC_PutMsgToExtQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_PutMsgToExtQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 	if((queManage = IPC_getQueManage(RPC_TRUE, queId)) != (IpcQueManage_t*)NULL) {
@@ -374,7 +377,7 @@ RpcResult_t IPC_PutMsgToExtQue(uint32_t queId, uint32_t msgId, RpcMsgParam_t *ms
 	}
 }
 
-RpcResult_t IPC_PutMsgToExtQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
+extern RpcResult_t IPC_PutMsgToExtQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam_t *msgParam, uint32_t timeout) {
 	IpcQueManage_t *queManage;
 
 	if((queManage = IPC_getQueManage(RPC_TRUE,queId)) != (IpcQueManage_t*)NULL) {
@@ -385,7 +388,7 @@ RpcResult_t IPC_PutMsgToExtQueUrgent(uint32_t queId, uint32_t msgId, RpcMsgParam
 	}
 }
 
-RpcResult_t IPC_GetNumOfMsgExt(uint32_t queId, uint32_t *numOfMsg) {
+extern RpcResult_t IPC_GetNumOfMsgExt(uint32_t queId, uint32_t *numOfMsg) {
 	IpcQueManage_t *queManage;
 
 	if(numOfMsg == (uint32_t*)NULL)
@@ -399,7 +402,7 @@ RpcResult_t IPC_GetNumOfMsgExt(uint32_t queId, uint32_t *numOfMsg) {
 	}
 }
 
-void IPC_PrintQueData(void) {
+extern void IPC_PrintQueData(void) {
 	IpcQueControl_t *gmqcp,*lmqcp;
 
 	gmqcp = IPC_getQueCtrl(RPC_TRUE);
@@ -505,7 +508,7 @@ static RpcResult_t IPC_createQueInternal(RpcBool_t IpcFlag,uint32_t queSize, uin
 		IPC_queFreeMem(IpcFlag,(void*)msgParamData);
 		return ret;
 	}
-	if((queManage = RPC_AllocListData(&(mqcp->queManageTop),sizeof(IpcQueManage_t),(uint32_t)IpcFlag,IPC_queAllocMem,IPC_queFreeMem)) == (IpcQueManage_t*)NULL) {
+	if((queManage = (IpcQueManage_t*)RPC_AllocListData(&(mqcp->queManageTop),sizeof(IpcQueManage_t),(uint32_t)IpcFlag,IPC_queAllocMem,IPC_queFreeMem)) == (IpcQueManage_t*)NULL) {
 		IPC_queFreeMem(IpcFlag,(void*)msgIdData);
 		IPC_queFreeMem(IpcFlag,(void*)msgParamData);
 		(void)RPC_MutexUnlock(&(mqcp->queCtrlMutex));
@@ -765,6 +768,9 @@ static RpcResult_t IPC_getMsgFromQueInernal(IpcQueManage_t *queManage, uint32_t
 		(void)RPC_MutexUnlock(&(queManage->queMutex));
 		return RPC_BUSY_ERROR;
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPushCleanup(IPC_msgQueCleanup,(void*)queManage);
+#endif	/* USE_CANCEL_TASK */
 	for(;;) {
 		if(queManage->numOfMsg > 0) {
 			break;
@@ -804,17 +810,30 @@ static RpcResult_t IPC_getMsgFromQueInernal(IpcQueManage_t *queManage, uint32_t
 			}
 		}
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPopCleanup(0);
+#endif	/* USE_CANCEL_TASK */
 	*msgId = queManage->msgIdData[queManage->queTop];
 	(void)memcpy((void*)msgParam,(void*)&(queManage->msgParamData[queManage->queTop]),sizeof(RpcMsgParam_t));
 	(queManage->numOfMsg)--;
 	queManage->queTop = ((queManage->queTop)+1)%(queManage->queSize);
+#if	0
 #ifdef	USE_DOUBLE_COND_IN_QUE
 	(void)RPC_CondSignal(&(queManage->fullCondId));
 #else	/* USE_DOUBLE_COND_IN_QUE */
-	(void)RPC_CondSignal(&(queManage->queCondId));
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
 #endif	/* USE_DOUBLE_COND_IN_QUE */
 	(queManage->useCounter)--;
 	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#else	/* 0 */
+	(queManage->useCounter)--;
+	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#ifdef	USE_DOUBLE_COND_IN_QUE
+	(void)RPC_CondSignal(&(queManage->fullCondId));
+#else	/* USE_DOUBLE_COND_IN_QUE */
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
+#endif	/* USE_DOUBLE_COND_IN_QUE */
+#endif	/* 0 */
 	return RPC_SUCCESS;
 }
 
@@ -833,12 +852,23 @@ static RpcResult_t IPC_sweepMsgFromQueInternal(IpcQueManage_t *queManage, uint32
 	(void)memcpy((void*)msgParam,(void*)&(queManage->msgParamData[queManage->queTop]),sizeof(RpcMsgParam_t));
 	(queManage->numOfMsg)--;
 	queManage->queTop = ((queManage->queTop)+1)%(queManage->queSize);
+#if	0
 #ifdef	USE_DOUBLE_COND_IN_QUE
 	(void)RPC_CondSignal(&(queManage->fullCondId));
 #else	/* USE_DOUBLE_COND_IN_QUE */
-	(queManage->useCounter)--;
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
 #endif	/* USE_DOUBLE_COND_IN_QUE */
+	(queManage->useCounter)--;
 	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#else	/* 0 */
+	(queManage->useCounter)--;
+	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#ifdef	USE_DOUBLE_COND_IN_QUE
+	(void)RPC_CondSignal(&(queManage->fullCondId));
+#else	/* USE_DOUBLE_COND_IN_QUE */
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
+#endif	/* USE_DOUBLE_COND_IN_QUE */
+#endif	/* 0 */
 	return RPC_SUCCESS;
 }
 
@@ -859,6 +889,9 @@ static RpcResult_t IPC_putMsgToQueInternal(IpcQueManage_t *queManage, uint32_t m
 		(void)RPC_MutexUnlock(&(queManage->queMutex));
 		return RPC_BUSY_ERROR;
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPushCleanup(IPC_msgQueCleanup,(void*)queManage);
+#endif	/* USE_CANCEL_TASK */
 	for(;;) {
 		if(queManage->numOfMsg < queManage->queSize) {
 			break;
@@ -898,6 +931,9 @@ static RpcResult_t IPC_putMsgToQueInternal(IpcQueManage_t *queManage, uint32_t m
 			}
 		}
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPopCleanup(0);
+#endif	/* USE_CANCEL_TASK */
 	queManage->msgIdData[queManage->queTail] = msgId;
 	if(msgParam == (RpcMsgParam_t*)NULL) {
 		(void)memset((void*)&(queManage->msgParamData[queManage->queTail]),0,sizeof(RpcMsgParam_t));
@@ -907,13 +943,23 @@ static RpcResult_t IPC_putMsgToQueInternal(IpcQueManage_t *queManage, uint32_t m
 	}
 	(queManage->numOfMsg)++;
 	queManage->queTail = ((queManage->queTail)+1)%(queManage->queSize);
+#if	0
 #ifdef	USE_DOUBLE_COND_IN_QUE
 	(void)RPC_CondSignal(&(queManage->emptyCondId));
 #else	/* USE_DOUBLE_COND_IN_QUE */
-	(void)RPC_CondSignal(&(queManage->queCondId));
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
 #endif	/* USE_DOUBLE_COND_IN_QUE */
 	(queManage->useCounter)--;
 	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#else	/* 0 */
+	(queManage->useCounter)--;
+	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#ifdef	USE_DOUBLE_COND_IN_QUE
+	(void)RPC_CondSignal(&(queManage->emptyCondId));
+#else	/* USE_DOUBLE_COND_IN_QUE */
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
+#endif	/* USE_DOUBLE_COND_IN_QUE */
+#endif	/* 0 */
 	return RPC_SUCCESS;
 }
 
@@ -934,6 +980,9 @@ static RpcResult_t IPC_putMsgToQueUrgentInternal(IpcQueManage_t *queManage, uint
 		(void)RPC_MutexUnlock(&(queManage->queMutex));
 		return RPC_BUSY_ERROR;
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPushCleanup(IPC_msgQueCleanup,(void*)queManage);
+#endif	/* USE_CANCEL_TASK */
 	for(;;) {
 		if(queManage->numOfMsg < queManage->queSize) {
 			break;
@@ -973,6 +1022,9 @@ static RpcResult_t IPC_putMsgToQueUrgentInternal(IpcQueManage_t *queManage, uint
 			}
 		}
 	}
+#ifdef	USE_CANCEL_TASK
+	RPC_TaskPopCleanup(0);
+#endif	/* USE_CANCEL_TASK */
 	if(queManage->queTop == 0)
 		queManage->queTop = queManage->queSize - 1;
 	else
@@ -985,13 +1037,23 @@ static RpcResult_t IPC_putMsgToQueUrgentInternal(IpcQueManage_t *queManage, uint
 		(void)memcpy((void*)&(queManage->msgParamData[queManage->queTop]),(void*)msgParam,sizeof(RpcMsgParam_t));
 	}
 	(queManage->numOfMsg)++;
+#if	0
 #ifdef	USE_DOUBLE_COND_IN_QUE
 	(void)RPC_CondSignal(&(queManage->emptyCondId));
 #else	/* USE_DOUBLE_COND_IN_QUE */
-	(void)RPC_CondSignal(&(queManage->queCondId));
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
 #endif	/* USE_DOUBLE_COND_IN_QUE */
 	(queManage->useCounter)--;
 	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#else	/* 0 */
+	(queManage->useCounter)--;
+	(void)RPC_MutexUnlock(&(queManage->queMutex));
+#ifdef	USE_DOUBLE_COND_IN_QUE
+	(void)RPC_CondSignal(&(queManage->emptyCondId));
+#else	/* USE_DOUBLE_COND_IN_QUE */
+	(void)RPC_CondBroadcast(&(queManage->queCondId));
+#endif	/* USE_DOUBLE_COND_IN_QUE */
+#endif	/* 0 */
 	return RPC_SUCCESS;
 }
 
@@ -1204,6 +1266,16 @@ static RpcBool_t IPC_compareQueInternalByQueId(RpcList_t *list,uint32_t queId) {
 	return RPC_FALSE;
 }
 
+#ifdef	USE_CANCEL_TASK
+static void IPC_msgQueCleanup(void *arg) {
+	IpcQueManage_t *queManage;
+
+	queManage = (IpcQueManage_t*)arg;
+	(queManage->useCounter)--;
+	(void)RPC_MutexUnlock(&(queManage->queMutex));
+}
+#endif	/* USE_CANCEL_TASK */
+
 #ifdef	IPC_USE_HASH_FUNC_FOR_MSG_QUE
 static RpcHashEntry_t *IPC_initMsgQueHash(RpcBool_t IpcFlag,uint32_t hashTableSize) {
 	RpcHashEntry_t *hashEntry;int i;
@@ -1338,7 +1410,8 @@ static RpcResult_t globalQueTest(void) {
 }
 
 static RpcResult_t localQueTest(void) {
-	RpcResult_t ret;uint32_t queId;RpcTaskID_t sendTask,recvTask;RpcTaskAttribute_t sendTaskAttr,recvTaskAttr;
+	RpcResult_t ret;static uint32_t queId;
+	RpcTaskID_t sendTask,recvTask;RpcTaskAttribute_t sendTaskAttr,recvTaskAttr;
 	struct timeval curTime;
 //	struct timeval sleepTime;
 	uint32_t errCode;
