@@ -41,12 +41,22 @@
 #include	"ipcLib.h"
 #include	"rpcFunc.h"
 #include	"rpcMsg.h"
+#include	"ipcMsg.h"
 #include	"ipcShm.h"
 #include	"ipcQue.h"
 #include	"ipcShmDB.h"
 #include	"ipcSkelton.h"
 #include	"ipcMarshal.h"
 
+typedef enum {
+	IPC_MSG_ID_SKELTON_LOCAL_UNKNOWN = IPC_MSG_ID_SKELTON_LOCAL_START,
+	IPC_MSG_ID_START_SKELTON,
+	IPC_MSG_ID_END_SKELTON,
+	IPC_MSG_ID_START_CANCEL,
+	IPC_MSG_ID_END_CANCEL,
+	IPC_MSG_ID_SKELTON_LOCAL_MAX
+}IpcSkeltonLocalMsg_t;
+
 typedef struct {
 	RpcTaskID_t taskId;
 	uint32_t queId;
@@ -84,8 +94,8 @@ static RpcResult_t IPC_allocSkeltonManagerResource(void);
 
 /* ----- Global Functions ----- */
 
-//RpcResult_t IPC_InitSkelton(IpcProcessType_t processType) {
-RpcResult_t IPC_InitSkelton(RpcBool_t masterProcess) {
+//extern RpcResult_t IPC_InitSkelton(IpcProcessType_t processType) {
+extern RpcResult_t IPC_InitSkelton(RpcBool_t masterProcess) {
 	RpcResult_t ret = RPC_SUCCESS;uint32_t pid;
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_SKELTON, RPC_LOG_LEVEL_DEBUG1, "IN IPC_InitSkelton PID:0x%x;",RPC_GetPid());
@@ -163,8 +173,8 @@ goError0:
 	return ret;
 }
 
-//RpcResult_t IPC_SkeltonRegisterFunction(uint32_t funcId,uint16_t accessDevice, uint16_t accessProcess,IPC_SkeltonFunc_t func,IPC_SkeltonCancelFunc_t cancelFunc) {
-RpcResult_t IPC_SkeltonRegisterFunction(uint32_t funcId,IPC_SkeltonFunc_t func,IPC_SkeltonCancelFunc_t cancelFunc) {
+//extern RpcResult_t IPC_SkeltonRegisterFunction(uint32_t funcId,uint16_t accessDevice, uint16_t accessProcess,IPC_SkeltonFunc_t func,IPC_SkeltonCancelFunc_t cancelFunc) {
+extern RpcResult_t IPC_SkeltonRegisterFunction(uint32_t funcId,IPC_SkeltonFunc_t func,IPC_SkeltonCancelFunc_t cancelFunc) {
 	RpcResult_t ret = RPC_SUCCESS;uint32_t pid;
 
 	pid = RPC_GetPid();
@@ -173,7 +183,7 @@ RpcResult_t IPC_SkeltonRegisterFunction(uint32_t funcId,IPC_SkeltonFunc_t func,I
 	return ret;
 }
 
-RpcResult_t IPC_SkeltonUnregisterFunction(uint32_t funcId) {
+extern RpcResult_t IPC_SkeltonUnregisterFunction(uint32_t funcId) {
 	RpcResult_t ret = RPC_SUCCESS;
 
 	ret = IPC_DeleteFuncDBEntry(funcId);
@@ -222,7 +232,7 @@ static void *IPC_skeltonManagerMain(void *arg1) {
 	uint32_t pid;
 	struct timeval sleepTime;
 
-	(void)RPC_SetCancelState(PTHREAD_CANCEL_ENABLE,NULL);
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,NULL);
 	(void)RPC_SetCancelType(PTHREAD_CANCEL_DEFERRED ,NULL);
 	queId = (uint32_t)(*(uint32_t*)arg1);
 	pid = (uint32_t)RPC_GetPid();
@@ -293,6 +303,8 @@ static void *IPC_skeltonTaskMain(void *arg1) {
 //	uint16_t accessProcessType;uint16_t accessDeviceType;
 	struct timeval sleepTime;
 
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,NULL);
+	(void)RPC_SetCancelType(PTHREAD_CANCEL_DEFERRED ,NULL);
 	pid = (uint32_t)RPC_GetPid();
 	queId = (uint32_t)(*(uint32_t*)arg1);
 	myTaskId = RPC_TaskSelf();
