@@ -41,6 +41,7 @@
 #include	"ipcLib.h"
 #include	"rpcFunc.h"
 #include	"rpcMsg.h"
+#include	"ipcMsg.h"
 #include	"ipcShm.h"
 #include	"ipcQue.h"
 #include	"ipcShmDB.h"
@@ -193,7 +194,7 @@ static RpcBool_t IPC_compareStubMltcstRetValBySeqNum(RpcList_t *list,uint32_t se
 
 /* ----- Global Functions ----- */
 
-RpcResult_t IPC_InitStub(RpcBool_t masterProcess) {
+extern RpcResult_t IPC_InitStub(RpcBool_t masterProcess) {
 	RpcResult_t ret = RPC_SUCCESS;
 
 #if	0	/* INIT LOG, INIT SHM, INIT SHM DB, INIT EXT PARAM, AND INIT QUE SYSTEM ARE IN MAIN */
@@ -265,11 +266,14 @@ goError0:
 	return ret;
 }
 
-RpcResult_t IPC_CallStubSync(uint32_t funcID, void *inData, uint32_t inSize, void **outData, uint32_t *outSize, uint32_t timeOut, uint32_t *errCode) {
+extern RpcResult_t IPC_CallStubSync(uint32_t funcID, void *inData, uint32_t inSize, void **outData, uint32_t *outSize, uint32_t timeOut, uint32_t *errCode) {
 	RpcResult_t ret = RPC_SUCCESS;uint32_t dstProcessId;RpcTaskID_t dstTaskId;uint32_t dstQueId;
 	RpcMsgParam_t msgParam;uint32_t srcProcessId;uint32_t srcQueId;uint32_t seqNum;
 	RpcMsgId_t msgId;struct timeval absTime;
 	IpcStubFuncInfo_t *funcInfo;RpcTaskID_t orgTaskId;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG1,"IN IPC_CallStubSync PID:0x%x; funcID:0x%x; inData:0x%x; inSize:%d; outData:0x%x; outSize:0x%x; timeOut:%d;",
 //		     RPC_GetPid(), funcID, inData, inSize, outData, outSize, timeOut);
@@ -279,6 +283,9 @@ RpcResult_t IPC_CallStubSync(uint32_t funcID, void *inData, uint32_t inSize, voi
 			IPC_FreeMarshaledSharedData(inData);
 		return RPC_PARAM_ERROR;
 	}
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
 	if(timeOut == 0) {
 		timeOut = IPC_STUB_DEFAULT_TIMEOUT;
 	}
@@ -389,6 +396,9 @@ RpcResult_t IPC_CallStubSync(uint32_t funcID, void *inData, uint32_t inSize, voi
 goOut:
 	(void)IPC_freeQueFromStubResourceDB(srcQueId);
 	IPC_deleteStubDBEntry(funcInfo);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG1,"OUT IPC_CallStubSync PID:0x%x; funcID:0x%x; outData:0x%x; outSize:%d;",
 //		RPC_GetPid(), funcID, *outData, *outSize);
 	return ret;
@@ -401,6 +411,9 @@ errorOut4:
 errorOut3:
 	if(inData != (void*)NULL)
 		IPC_FreeMarshaledSharedData(inData);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 	return ret;
 
 errorOut2:
@@ -411,15 +424,21 @@ errorOut1:
 	*outData = (void*)NULL;*outSize = 0;
 	(void)IPC_freeQueFromStubResourceDB(srcQueId);
 	IPC_deleteStubDBEntry(funcInfo);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 	return ret;
 }
 
-RpcResult_t IPC_CallStubAsync(uint32_t funcID, void *inData, uint32_t inSize, uint32_t timeOut, IPC_CallbackFunc_t callbackFunc, uint32_t *seqNum,uint32_t *errCode) {
+extern RpcResult_t IPC_CallStubAsync(uint32_t funcID, void *inData, uint32_t inSize, uint32_t timeOut, IPC_CallbackFunc_t callbackFunc, uint32_t *seqNum,uint32_t *errCode) {
 	RpcResult_t ret = RPC_SUCCESS;uint32_t dstProcessId;RpcTaskID_t dstTaskId;uint32_t dstQueId;
 	RpcMsgParam_t msgParam;uint32_t srcProcessId;uint32_t srcQueId;
 	RpcMsgId_t msgId;struct timeval absTime;
 	IpcStubFuncInfo_t *funcInfo;RpcTaskID_t orgTaskId;
 	RpcTaskID_t cbTaskId;RpcTaskControl_t *tskCntl;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG1,"IN IPC_CallStubAsync PID:0x%x; funcID:0x%x;, inData:0x%x; inSize:%d; timeOut:%d; callbackFunc:0x%x;",
 //		     RPC_GetPid(), funcID, inData, inSize, timeOut, callbackFunc);
@@ -429,6 +448,9 @@ RpcResult_t IPC_CallStubAsync(uint32_t funcID, void *inData, uint32_t inSize, ui
 			IPC_FreeMarshaledSharedData(inData);
 		return RPC_PARAM_ERROR;
 	}
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
 	if(timeOut == 0) {
 		timeOut = IPC_STUB_DEFAULT_TIMEOUT;
 	}
@@ -533,6 +555,9 @@ goOut:
 	if(msgParam.u.msgParam.extParam != (void*)NULL) {
 		IPC_FreeMarshaledSharedData(msgParam.u.msgParam.extParam);
 	}
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 //	(void)IPC_freeQueFromStubResourceDB(srcQueId);
 //	IPC_deleteStubDBEntry(funcInfo);
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG1,"OUT IPC_CallStubAsync PID:0x%x; funcID:0x%x;",
@@ -549,6 +574,9 @@ errorOut3:
 	*seqNum = 0;
 	if(inData != (void*)NULL)
 		IPC_FreeMarshaledSharedData(inData);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 	return ret;
 
 errorOut2:
@@ -560,11 +588,18 @@ errorOut1:
 	(void)IPC_freeQueFromStubResourceDB(srcQueId);
 	(void)IPC_freeCBTaskFromStubResourceDB(cbTaskId);
 	IPC_deleteStubDBEntry(funcInfo);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
 	return ret;
 }
 
-RpcResult_t IPC_StubSyncCancel(RpcTaskID_t taskId,uint32_t *errCode) {
+extern RpcResult_t IPC_StubSyncCancel(RpcTaskID_t taskId,uint32_t *errCode) {
 	uint32_t seqNum;IpcStubFuncInfo_t *funcInfo;uint32_t funcId;
+	RpcResult_t ret;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 	if((funcInfo = IPC_searchStubDBEntryByOrgTaskId(taskId)) == (IpcStubFuncInfo_t*)NULL) {
 		*errCode = RPC_ERR_OTHER;
@@ -572,11 +607,21 @@ RpcResult_t IPC_StubSyncCancel(RpcTaskID_t taskId,uint32_t *errCode) {
 	}
 	seqNum = funcInfo->seqNum;
 	funcId = funcInfo->funcId;
-	return IPC_execCancelFunc(funcId,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
+	ret = IPC_execCancelFunc(funcId,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
+	return ret;
 }
 
-RpcResult_t IPC_StbuAsyncCancel(uint32_t seqNum,uint32_t *errCode) {
-	IpcStubFuncInfo_t *funcInfo;uint32_t funcId;
+extern RpcResult_t IPC_StbuAsyncCancel(uint32_t seqNum,uint32_t *errCode) {
+	IpcStubFuncInfo_t *funcInfo;uint32_t funcId;RpcResult_t ret;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 	if((funcInfo = IPC_searchStubDBEntryBySeqNum(seqNum)) == (IpcStubFuncInfo_t*)NULL) {
 		*errCode = RPC_ERR_OTHER;
@@ -584,12 +629,22 @@ RpcResult_t IPC_StbuAsyncCancel(uint32_t seqNum,uint32_t *errCode) {
 	}
 	seqNum = funcInfo->seqNum;
 	funcId = funcInfo->funcId;
-	return IPC_execCancelFunc(funcId,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
+	ret = IPC_execCancelFunc(funcId,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
+	return ret;
 }
 
 #ifdef	USE_MULTICAST_RPC
-RpcResult_t IPC_CallStubSyncMulticast(uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t *numOfRet,IpcMulticastRetData_t **retData,uint32_t timeOut,uint32_t *errCode) {
-	RpcTaskID_t taskId;uint32_t numOfCalls = 0;
+extern RpcResult_t IPC_CallStubSyncMulticast(uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t *numOfRet,IpcMulticastRetData_t **retData,uint32_t timeOut,uint32_t *errCode) {
+	RpcTaskID_t taskId;uint32_t numOfCalls = 0;RpcResult_t ret;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 	if((retData == (IpcMulticastRetData_t**)NULL)||(numOfRet == (uint32_t*)NULL)||(errCode == (uint32_t*)NULL)) {
 		if(inParams != (void*)NULL)
@@ -600,11 +655,21 @@ RpcResult_t IPC_CallStubSyncMulticast(uint32_t mltcstFuncId,void *inParams,uint3
 		timeOut = IPC_STUB_DEFAULT_TIMEOUT;
 	}
 	taskId = RPC_TaskSelf();
-	return IPC_callSyncMulticastFunction(taskId,mltcstFuncId,inParams, inParamsSize, &numOfCalls,numOfRet,retData,timeOut,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
+	ret = IPC_callSyncMulticastFunction(taskId,mltcstFuncId,inParams, inParamsSize, &numOfCalls,numOfRet,retData,timeOut,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
+	return ret;
 }
 
-RpcResult_t IPC_CallStubAsyncMulticast(uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t timeOut,IPC_MultticastCallbackFunc_t mltcstCallback, uint32_t *seqNum, uint32_t *errCode) {
-	RpcTaskID_t taskId;uint32_t numOfCalls = 0;
+extern RpcResult_t IPC_CallStubAsyncMulticast(uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t timeOut,IPC_MultticastCallbackFunc_t mltcstCallback, uint32_t *seqNum, uint32_t *errCode) {
+	RpcTaskID_t taskId;uint32_t numOfCalls = 0;RpcResult_t ret;
+#ifdef	USE_CANCEL_TASK
+	int oldState;
+#endif	/* USE_CANCEL_TASK */
 
 	if((mltcstCallback == NULL)||(seqNum == (uint32_t*)NULL)||(errCode == (uint32_t*)NULL)) {
 		if(inParams != (void*)NULL)
@@ -615,10 +680,17 @@ RpcResult_t IPC_CallStubAsyncMulticast(uint32_t mltcstFuncId,void *inParams,uint
 		timeOut = IPC_STUB_DEFAULT_TIMEOUT;
 	}
 	taskId = RPC_TaskSelf();
-	return IPC_callAsyncMulticastFunction(taskId,mltcstFuncId,inParams,inParamsSize,&numOfCalls,timeOut, mltcstCallback,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,&oldState);
+#endif	/* USE_CANCEL_TASK */
+	ret = IPC_callAsyncMulticastFunction(taskId,mltcstFuncId,inParams,inParamsSize,&numOfCalls,timeOut, mltcstCallback,seqNum,errCode);
+#ifdef	USE_CANCEL_TASK
+	(void)RPC_SetCancelState(oldState,(int*)NULL);
+#endif	/* USE_CANCEL_TASK */
+	return ret;
 }
 
-void IPC_FreeMulticastOutParams(IpcMulticastRetData_t *retData) {
+extern void IPC_FreeMulticastOutParams(IpcMulticastRetData_t *retData) {
 	if(retData != (IpcMulticastRetData_t*)NULL)
 		RPC_FreeMem((void*)retData);
 }
@@ -654,7 +726,7 @@ static RpcResult_t IPC_initStubResourceDB(void) {
 		return RPC_FATAL_ERROR;
 	}
 	for(i = 0;i < IPC_STUB_START_NUM_OF_QUE_RESOURCE;i++) {
-		if((qEntry = RPC_AllocListData(&(IpcStubRcsDBManage.stubQueResourceList),sizeof(IpcStubQueResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubQueResourceDBEntry_t*)NULL) {
+		if((qEntry = (IpcStubQueResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubQueResourceList),sizeof(IpcStubQueResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubQueResourceDBEntry_t*)NULL) {
 			return RPC_NO_MORE_RESOURCE;
 		}
 //		(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG3,"  ALLOCATED LIST DATA FOR STUB QUE  LIST-TOP:0x%x;",IpcStubRcsDBManage.stubQueResourceList);
@@ -663,7 +735,7 @@ static RpcResult_t IPC_initStubResourceDB(void) {
 		}
 	}
 	for(i = 0;i < IPC_STUB_START_NUM_OF_CB_TASK_RESOURCE;i++) {
-		if((tEntry = RPC_AllocListData(&(IpcStubRcsDBManage.stubCBTaskResourceList),sizeof(IpcStubCBTaskResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubCBTaskResourceDBEntry_t*)NULL) {
+		if((tEntry = (IpcStubCBTaskResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubCBTaskResourceList),sizeof(IpcStubCBTaskResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubCBTaskResourceDBEntry_t*)NULL) {
 			return RPC_NO_MORE_RESOURCE;
 		}
 //		(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG3,"  ALLOCATED LIST DATA FOR STUB CB LIST-TOP:0x%x;",IpcStubRcsDBManage.stubCBTaskResourceList);
@@ -673,7 +745,7 @@ static RpcResult_t IPC_initStubResourceDB(void) {
 	}
 #ifdef	USE_MULTICAST_RPC
 	for(i = 0;i < IPC_STUB_START_NUM_OF_MLTCST_RESOURCE;i++) {
-		if((mEntry = RPC_AllocListData(&(IpcStubRcsDBManage.stubMltcstResourceList),sizeof(IpcStubMltcstResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubMltcstResourceDBEntry_t*)NULL) {
+		if((mEntry = (IpcStubMltcstResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubMltcstResourceList),sizeof(IpcStubMltcstResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubMltcstResourceDBEntry_t*)NULL) {
 			return RPC_NO_MORE_RESOURCE;
 		}
 //		(void)RPC_LogPrint(RPC_LOG_MODULE_STUB,RPC_LOG_LEVEL_DEBUG3,"  ALLOCATED LIST DATA FOR STUB MLTCST RCS LIST-TOP:0x%x;",IpcStubRcsDBManage.stubMltcstResourceList);
@@ -721,6 +793,8 @@ static void *IPC_CBTaskMain(void *arg1) {
 
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_STUB, RPC_LOG_LEVEL_DEBUG1,"IN IPC_CBTaskMain PID:0x%x; arg1:0x%x;",RPC_GetPid(),arg1);
 //	pid = (uint32_t)RPC_GetPid();
+	(void)RPC_SetCancelState(PTHREAD_CANCEL_DISABLE,(int*)NULL);
+	(void)RPC_SetCancelType(PTHREAD_CANCEL_DEFERRED ,NULL);
 	tskCntl = (RpcTaskControl_t*)(arg1);
 	myTaskId = RPC_TaskSelf();
 	for(;;) {
@@ -1019,7 +1093,7 @@ static IpcStubCBTaskResourceDBEntry_t *IPC_getIdleStubCBTaskResource(void) {
 		(void)RPC_MutexUnlock(&(IpcStubDataManage.mutexId));
 		return (IpcStubCBTaskResourceDBEntry_t*)NULL;
 	}
-	if((new = RPC_AllocListData(&(IpcStubRcsDBManage.stubCBTaskResourceList),sizeof(IpcStubCBTaskResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) != (IpcStubCBTaskResourceDBEntry_t*)NULL) {
+	if((new = (IpcStubCBTaskResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubCBTaskResourceList),sizeof(IpcStubCBTaskResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) != (IpcStubCBTaskResourceDBEntry_t*)NULL) {
 		(void)RPC_MutexUnlock(&(IpcStubDataManage.mutexId));
 		return (IpcStubCBTaskResourceDBEntry_t*)NULL;			
 	}
@@ -1084,7 +1158,7 @@ static IpcStubQueResourceDBEntry_t *IPC_getIdleStubQueResource(void) {
 		(void)RPC_MutexUnlock(&(IpcStubDataManage.mutexId));
 		return (IpcStubQueResourceDBEntry_t*)NULL;
 	}
-	if((new = RPC_AllocListData(&(IpcStubRcsDBManage.stubQueResourceList),sizeof(IpcStubQueResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubQueResourceDBEntry_t*)NULL) {
+	if((new = (IpcStubQueResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubQueResourceList),sizeof(IpcStubQueResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) == (IpcStubQueResourceDBEntry_t*)NULL) {
 		(void)RPC_MutexUnlock(&(IpcStubDataManage.mutexId));
 		return (IpcStubQueResourceDBEntry_t*)NULL;
 	}
@@ -1237,7 +1311,7 @@ static RpcResult_t IPC_initStubMltcstCntl(void) {
 }
 
 static RpcResult_t IPC_callSyncMulticastFunction(RpcTaskID_t callTaskId,uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t *numOfCalls,uint32_t *numOfRet,IpcMulticastRetData_t **retData,uint32_t timeOut,uint32_t *errCode) {
-	RpcResult_t ret;RpcList_t *funcList;IpcStubMltcstResourceDBEntry_t *mltcstRcs;
+	RpcResult_t ret;RpcList_t *funcList = (RpcList_t*)NULL;IpcStubMltcstResourceDBEntry_t *mltcstRcs;
 	IpcFuncRetVal_t *retValArea;IpcMltcstRetCntl_t *mltcstRetCntl;RpcList_t *list;
 	IpcMltcstFuncEntry_t *funcEntry;uint32_t seqNum;IpcMulticastRetData_t *retVal,*p;
 	struct timeval absTime;struct timespec specTime;void *dupData;int32_t calls = 0;
@@ -1357,6 +1431,8 @@ static RpcResult_t IPC_callSyncMulticastFunction(RpcTaskID_t callTaskId,uint32_t
 		IPC_FreeMarshaledSharedData(inParams);
 		inParams = (void*)NULL;
 	}
+	IPC_FreeFuncList(funcList);
+	funcList = (RpcList_t*)NULL;
 	mltcstRetCntl->numOfCall = *numOfCalls;
 	if(!RPC_CheckTimeout(&absTime)) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_MLTCST,RPC_LOG_LEVEL_ERROR, " TIMEOUT OCCUR");
@@ -1369,6 +1445,7 @@ static RpcResult_t IPC_callSyncMulticastFunction(RpcTaskID_t callTaskId,uint32_t
 	RPC_ConvertTime(&absTime,&specTime);
 	if((ret = RPC_TaskControlTimedWait(&(mltcstRcs->tskCntl),&specTime)) != RPC_SUCCESS) {
 		if(ret != RPC_TIMEOUT_ERROR) {
+			(void)RPC_MutexLock(&(IpcStubMltcstCntl.mutexId));
 			goto OccrError1;
 		}
 	}
@@ -1395,7 +1472,6 @@ static RpcResult_t IPC_callSyncMulticastFunction(RpcTaskID_t callTaskId,uint32_t
 		RPC_FreeListData(&(mltcstRetCntl->funcRetVals),(void*)(list->entry),0,RPC_LclFreeMemWrapper);
 	}
 	IPC_deleteStubMltcstCntlEntry(mltcstRetCntl);
-	IPC_FreeFuncList(funcList);
 	(void)RPC_MutexUnlock(&(IpcStubMltcstCntl.mutexId));
 	*retData = (IpcMulticastRetData_t*)retVal;
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_MLTCST,RPC_LOG_LEVEL_DEBUG1,"OUT IPC_callSyncMulticastFunction numOfRet:%d; retData:0x%x;",
@@ -1423,7 +1499,7 @@ OccrError2:
 }
 
 static RpcResult_t IPC_callAsyncMulticastFunction(RpcTaskID_t callTaskId,uint32_t mltcstFuncId,void *inParams,uint32_t inParamsSize,uint32_t *numOfCalls,uint32_t timeOut,IPC_MultticastCallbackFunc_t mltcstCallback,uint32_t *seqNum,uint32_t *errCode) {
-	RpcResult_t ret;RpcList_t *funcList;
+	RpcResult_t ret;RpcList_t *funcList = (RpcList_t*)NULL;
 	IpcFuncRetVal_t *retValArea;IpcMltcstRetCntl_t *mltcstRetCntl;RpcList_t *list;
 	IpcMltcstFuncEntry_t *funcEntry;uint32_t lclSeqNum;void *dupData;int32_t calls = 0;
 	struct timeval absTime;
@@ -1524,6 +1600,8 @@ static RpcResult_t IPC_callAsyncMulticastFunction(RpcTaskID_t callTaskId,uint32_
 		IPC_FreeMarshaledSharedData(inParams);
 		inParams = (void*)NULL;
 	}
+	IPC_FreeFuncList(funcList);
+	funcList = (RpcList_t*)NULL;
 	mltcstRetCntl->numOfCall = *numOfCalls;
 	if(!RPC_CheckTimeout(&absTime)) {
 		(void)RPC_LogPrint(RPC_LOG_MODULE_MLTCST,RPC_LOG_LEVEL_ERROR, " TIMEOUT OCCUR");
@@ -1533,7 +1611,6 @@ static RpcResult_t IPC_callAsyncMulticastFunction(RpcTaskID_t callTaskId,uint32_
 	}
 	*seqNum = IPC_assignNewSequenceNubmer();
 	mltcstRetCntl->seqNum = *seqNum;
-	IPC_FreeFuncList(funcList);
 	(void)RPC_MutexUnlock(&(IpcStubMltcstCntl.mutexId));
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_MLTCST,RPC_LOG_LEVEL_DEBUG1,"OUT IPC_callAsyncMulticastFunction seqNum:0x%d; numOfCalls:%d;",*seqNum,*numOfCalls);
 	return RPC_SUCCESS;
@@ -1738,7 +1815,7 @@ static IpcStubMltcstResourceDBEntry_t *IPC_getIdleStubMltcstResource(void) {
 	if(num >= IPC_STUB_START_NUM_OF_MLTCST_RESOURCE) {
 		return (IpcStubMltcstResourceDBEntry_t*)NULL;
 	}
-	if((new = RPC_AllocListData(&(IpcStubRcsDBManage.stubMltcstResourceList),sizeof(IpcStubMltcstResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) != (IpcStubMltcstResourceDBEntry_t*)NULL) {
+	if((new = (IpcStubMltcstResourceDBEntry_t*)RPC_AllocListData(&(IpcStubRcsDBManage.stubMltcstResourceList),sizeof(IpcStubMltcstResourceDBEntry_t),0,RPC_LclAllocMemWrapper,RPC_LclFreeMemWrapper)) != (IpcStubMltcstResourceDBEntry_t*)NULL) {
 		return (IpcStubMltcstResourceDBEntry_t*)NULL;			
 	}
 	if((ret = IPC_allocStubMltcstResource(new)) != RPC_SUCCESS) {
