@@ -46,7 +46,7 @@ static void RPC_convertErrno(int num, uint32_t *errCode);
 
 /*--- Task APIs ---*/
 
-RpcResult_t RPC_CreateTask(RpcTaskID_t *taskID, RpcTaskAttribute_t *taskAttr, RPC_TaskStartRoutine_t startRoutine, void *arg) {
+extern RpcResult_t RPC_CreateTask(RpcTaskID_t *taskID, RpcTaskAttribute_t *taskAttr, RPC_TaskStartRoutine_t startRoutine, void *arg) {
 	int err; RpcResult_t ret;
 
 	if((err = pthread_create((pthread_t*)taskID, (pthread_attr_t*)taskAttr, startRoutine, arg)) != 0) {
@@ -57,11 +57,11 @@ RpcResult_t RPC_CreateTask(RpcTaskID_t *taskID, RpcTaskAttribute_t *taskAttr, RP
 	return RPC_SUCCESS;
 }
 
-void RPC_ExitTask(void *retVal) {
+extern void RPC_ExitTask(void *retVal) {
 	pthread_exit(retVal);
 }
 
-RpcResult_t RPC_JoinTask(RpcTaskID_t taskID, void **retVal) {
+extern RpcResult_t RPC_JoinTask(RpcTaskID_t taskID, void **retVal) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_join((pthread_t)taskID,retVal)) != 0) {
@@ -72,7 +72,7 @@ RpcResult_t RPC_JoinTask(RpcTaskID_t taskID, void **retVal) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_DetachTask(RpcTaskID_t taskID) {
+extern RpcResult_t RPC_DetachTask(RpcTaskID_t taskID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_detach((pthread_t)taskID)) != 0) {
@@ -82,7 +82,7 @@ RpcResult_t RPC_DetachTask(RpcTaskID_t taskID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CancelTask(RpcTaskID_t taskID) {
+extern RpcResult_t RPC_CancelTask(RpcTaskID_t taskID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cancel((pthread_t)taskID)) != 0) {
@@ -92,7 +92,7 @@ RpcResult_t RPC_CancelTask(RpcTaskID_t taskID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SetCancelState(int state, int *oldState) {
+extern RpcResult_t RPC_SetCancelState(int state, int *oldState) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_setcancelstate(state,oldState)) != 0) {
@@ -102,7 +102,7 @@ RpcResult_t RPC_SetCancelState(int state, int *oldState) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SetCancelType(int type, int *oldType) {
+extern RpcResult_t RPC_SetCancelType(int type, int *oldType) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_setcanceltype(type, oldType)) != 0) {
@@ -112,32 +112,32 @@ RpcResult_t RPC_SetCancelType(int type, int *oldType) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TestCancel(void) {
+extern RpcResult_t RPC_TestCancel(void) {
 	pthread_testcancel();
 	return RPC_SUCCESS;
 }
 
-RpcTaskID_t RPC_TaskSelf(void) {
+extern RpcTaskID_t RPC_TaskSelf(void) {
 	return (RpcTaskID_t)pthread_self();
 }
 
 /*
-RpcResult_t RPC_TaskPushCleanup(void (*routine)(void*), void *arg) {
+extern RpcResult_t RPC_TaskPushCleanup(void (*routine)(void*), void *arg) {
 	pthread_cleanup_push(routine, arg);
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskPopCleanup(int execute) {
+extern RpcResult_t RPC_TaskPopCleanup(int execute) {
 	pthread_cleanup_pop(execute);
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskPushdClenaupDeferNP(void (*routine)(void*), void *arg) {
+extern RpcResult_t RPC_TaskPushdClenaupDeferNP(void (*routine)(void*), void *arg) {
 	pthread_cleanup_push_defer_np(routine, arg);
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskPopCleanupRestoreNP(int execute) {
+extern RpcResult_t RPC_TaskPopCleanupRestoreNP(int execute) {
 	pthread_cleanup_pop_defer_np(execute);
 	return RPC_SUCCESS;
 }
@@ -145,7 +145,7 @@ RpcResult_t RPC_TaskPopCleanupRestoreNP(int execute) {
 
 /*--- Task Attribute APIs ---*/
 
-RpcResult_t RPC_TaskAttrInit(RpcTaskAttribute_t *taskAttr) {
+extern RpcResult_t RPC_TaskAttrInit(RpcTaskAttribute_t *taskAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_init((pthread_attr_t*)taskAttr)) != 0) {
@@ -155,7 +155,7 @@ RpcResult_t RPC_TaskAttrInit(RpcTaskAttribute_t *taskAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetDetachState(RpcTaskAttribute_t *taskAttr, int detachState) {
+extern RpcResult_t RPC_TaskAttrSetDetachState(RpcTaskAttribute_t *taskAttr, int detachState) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_setdetachstate((pthread_attr_t*)taskAttr, detachState)) != 0) {
@@ -165,7 +165,7 @@ RpcResult_t RPC_TaskAttrSetDetachState(RpcTaskAttribute_t *taskAttr, int detachS
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrGetDetachState(RpcTaskAttribute_t *taskAttr, int *detachState) {
+extern RpcResult_t RPC_TaskAttrGetDetachState(RpcTaskAttribute_t *taskAttr, int *detachState) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getdetachstate((pthread_attr_t*)taskAttr, detachState)) != 0) {
@@ -176,7 +176,7 @@ RpcResult_t RPC_TaskAttrGetDetachState(RpcTaskAttribute_t *taskAttr, int *detach
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetSchedPolicy(RpcTaskAttribute_t *taskAttr, int policy) {
+extern RpcResult_t RPC_TaskAttrSetSchedPolicy(RpcTaskAttribute_t *taskAttr, int policy) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_setschedpolicy((pthread_attr_t*)taskAttr,policy)) != 0) {
@@ -186,7 +186,7 @@ RpcResult_t RPC_TaskAttrSetSchedPolicy(RpcTaskAttribute_t *taskAttr, int policy)
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrGetSchedPolicy(RpcTaskAttribute_t *taskAttr, int *policy) {
+extern RpcResult_t RPC_TaskAttrGetSchedPolicy(RpcTaskAttribute_t *taskAttr, int *policy) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getschedpolicy((pthread_attr_t*)taskAttr, policy)) != 0) {
@@ -197,7 +197,7 @@ RpcResult_t RPC_TaskAttrGetSchedPolicy(RpcTaskAttribute_t *taskAttr, int *policy
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSchedParam_t *schedParam) {
+extern RpcResult_t RPC_TaskAttrSetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSchedParam_t *schedParam) {
 	RpcResult_t ret; int err;
 
 	if((err = pthread_attr_setschedparam((pthread_attr_t*)taskAttr, (struct sched_param*)schedParam)) != 0) {
@@ -206,7 +206,7 @@ RpcResult_t RPC_TaskAttrSetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSched
 	}
 	return RPC_SUCCESS;
 }
-RpcResult_t RPC_TaskAttrGetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSchedParam_t *schedParam) {
+extern RpcResult_t RPC_TaskAttrGetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSchedParam_t *schedParam) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getschedparam((pthread_attr_t*)taskAttr, (struct sched_param*)schedParam)) != 0) {
@@ -216,7 +216,7 @@ RpcResult_t RPC_TaskAttrGetSchedParam(RpcTaskAttribute_t *taskAttr, RpcTaskSched
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetInheritSched(RpcTaskAttribute_t *taskAttr, int inherit) {
+extern RpcResult_t RPC_TaskAttrSetInheritSched(RpcTaskAttribute_t *taskAttr, int inherit) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_setinheritsched((pthread_attr_t*)taskAttr, inherit)) != 0) {
@@ -226,7 +226,7 @@ RpcResult_t RPC_TaskAttrSetInheritSched(RpcTaskAttribute_t *taskAttr, int inheri
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrGetInheritSched(RpcTaskAttribute_t *taskAttr, int *inherit) {
+extern RpcResult_t RPC_TaskAttrGetInheritSched(RpcTaskAttribute_t *taskAttr, int *inherit) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getinheritsched((pthread_attr_t*)taskAttr, inherit)) != 0) {
@@ -237,7 +237,7 @@ RpcResult_t RPC_TaskAttrGetInheritSched(RpcTaskAttribute_t *taskAttr, int *inher
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrDestroy(RpcTaskAttribute_t *taskAttr) {
+extern RpcResult_t RPC_TaskAttrDestroy(RpcTaskAttribute_t *taskAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_destroy(taskAttr)) != 0) {
@@ -247,7 +247,7 @@ RpcResult_t RPC_TaskAttrDestroy(RpcTaskAttribute_t *taskAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetStackSize(RpcTaskAttribute_t *taskAttr, int stackSize) {
+extern RpcResult_t RPC_TaskAttrSetStackSize(RpcTaskAttribute_t *taskAttr, int stackSize) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_setstacksize((pthread_attr_t*)taskAttr, stackSize)) != 0) {
@@ -257,7 +257,7 @@ RpcResult_t RPC_TaskAttrSetStackSize(RpcTaskAttribute_t *taskAttr, int stackSize
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrGetStackSize(RpcTaskAttribute_t *taskAttr, int *stackSize) {
+extern RpcResult_t RPC_TaskAttrGetStackSize(RpcTaskAttribute_t *taskAttr, int *stackSize) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getstacksize(taskAttr, stackSize)) != 0) {
@@ -268,7 +268,7 @@ RpcResult_t RPC_TaskAttrGetStackSize(RpcTaskAttribute_t *taskAttr, int *stackSiz
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrSetStack(RpcTaskAttribute_t *taskAttr, void *stackAddr, int stackSize) {
+extern RpcResult_t RPC_TaskAttrSetStack(RpcTaskAttribute_t *taskAttr, void *stackAddr, int stackSize) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_setstack((pthread_attr_t*)taskAttr, stackAddr, stackSize)) != 0) {
@@ -278,7 +278,7 @@ RpcResult_t RPC_TaskAttrSetStack(RpcTaskAttribute_t *taskAttr, void *stackAddr,
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_TaskAttrGetStack(RpcTaskAttribute_t *taskAttr, void **stackAddr, int *stackSize) {
+extern RpcResult_t RPC_TaskAttrGetStack(RpcTaskAttribute_t *taskAttr, void **stackAddr, int *stackSize) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_attr_getstack((pthread_attr_t*)taskAttr, stackAddr, stackSize)) != 0) {
@@ -292,7 +292,7 @@ RpcResult_t RPC_TaskAttrGetStack(RpcTaskAttribute_t *taskAttr, void **stackAddr,
 
 /*--- Task Schedule APIs ---*/
 
-RpcResult_t RPC_TaskYield(void) {
+extern RpcResult_t RPC_TaskYield(void) {
 #ifdef	__USE_GNU
 	{
 	RpcResult_t ret; int err;
@@ -316,7 +316,7 @@ RpcResult_t RPC_TaskYield(void) {
 #endif	/* __USE_GNU */
 }
 
-RpcResult_t RPC_TaskSleep(struct timeval *sleepTime,struct timeval *remainTime,uint32_t *errCode) {
+extern RpcResult_t RPC_TaskSleep(struct timeval *sleepTime,struct timeval *remainTime,uint32_t *errCode) {
 	struct timespec req,rem;
 
 	if(remainTime != (struct timeval*)NULL) {
@@ -350,7 +350,7 @@ RpcResult_t RPC_TaskSleep(struct timeval *sleepTime,struct timeval *remainTime,u
 
 /*--- Mutex APIs ---*/
 
-RpcResult_t RPC_InitMutex(RpcMutexID_t *mutexID, RpcMutexAttribute_t *mutexAttr) {
+extern RpcResult_t RPC_InitMutex(RpcMutexID_t *mutexID, RpcMutexAttribute_t *mutexAttr) {
 	RpcResult_t ret; int err;
 
 	if((err = pthread_mutex_init((pthread_mutex_t*)mutexID, (pthread_mutexattr_t*)mutexAttr)) != 0) {
@@ -361,7 +361,7 @@ RpcResult_t RPC_InitMutex(RpcMutexID_t *mutexID, RpcMutexAttribute_t *mutexAttr)
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexLock(RpcMutexID_t *mutexID) {
+extern RpcResult_t RPC_MutexLock(RpcMutexID_t *mutexID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutex_lock((pthread_mutex_t*)mutexID)) != 0) {
@@ -370,7 +370,8 @@ RpcResult_t RPC_MutexLock(RpcMutexID_t *mutexID) {
 	}
 	return RPC_SUCCESS;
 }
-RpcResult_t RPC_MutexUnlock(RpcMutexID_t *mutexID) {
+
+extern RpcResult_t RPC_MutexUnlock(RpcMutexID_t *mutexID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutex_unlock((pthread_mutex_t*)mutexID)) != 0) {
@@ -380,7 +381,7 @@ RpcResult_t RPC_MutexUnlock(RpcMutexID_t *mutexID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexLockTry(RpcMutexID_t *mutexID) {
+extern RpcResult_t RPC_MutexLockTry(RpcMutexID_t *mutexID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutex_trylock((pthread_mutex_t*)mutexID)) != 0) {
@@ -390,7 +391,7 @@ RpcResult_t RPC_MutexLockTry(RpcMutexID_t *mutexID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexDestroy(RpcMutexID_t *mutexID) {
+extern RpcResult_t RPC_MutexDestroy(RpcMutexID_t *mutexID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutex_destroy((pthread_mutex_t*)mutexID)) != 0) {
@@ -400,9 +401,19 @@ RpcResult_t RPC_MutexDestroy(RpcMutexID_t *mutexID) {
 	return RPC_SUCCESS;
 }
 
+extern RpcResult_t RPC_MutexTimedLock(RpcMutexID_t *mutexID,struct timespec *absTime) {
+	RpcResult_t ret;int err;
+
+	if((err = pthread_mutex_timedlock((pthread_mutex_t*)mutexID,absTime)) != 0) {
+		RPC_convertErrCode(err, &ret);
+		return ret;
+	}
+	return RPC_SUCCESS;
+}
+
 /*--- Mutex Attribute APIs ---*/
 
-RpcResult_t RPC_MutexAttrInit(RpcMutexAttribute_t *mutexAttr) {
+extern RpcResult_t RPC_MutexAttrInit(RpcMutexAttribute_t *mutexAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_init((pthread_mutexattr_t*)mutexAttr)) != 0) {
@@ -412,7 +423,7 @@ RpcResult_t RPC_MutexAttrInit(RpcMutexAttribute_t *mutexAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexAttrSetType(RpcMutexAttribute_t *mutexAttr, int kind) {
+extern RpcResult_t RPC_MutexAttrSetType(RpcMutexAttribute_t *mutexAttr, int kind) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_settype(mutexAttr, kind)) != 0) {
@@ -422,7 +433,7 @@ RpcResult_t RPC_MutexAttrSetType(RpcMutexAttribute_t *mutexAttr, int kind) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexAttrGetType(RpcMutexAttribute_t *mutexAttr, int *kind) {
+extern RpcResult_t RPC_MutexAttrGetType(RpcMutexAttribute_t *mutexAttr, int *kind) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_gettype(mutexAttr, kind)) != 0) {
@@ -437,7 +448,7 @@ RpcResult_t RPC_MutexAttrGetType(RpcMutexAttribute_t *mutexAttr, int *kind) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexAttrDestroy(RpcMutexAttribute_t *mutexAttr) {
+extern RpcResult_t RPC_MutexAttrDestroy(RpcMutexAttribute_t *mutexAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_destroy(mutexAttr)) != 0) {
@@ -447,7 +458,7 @@ RpcResult_t RPC_MutexAttrDestroy(RpcMutexAttribute_t *mutexAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexAttrGetPshared(RpcMutexAttribute_t *mutexAttr, int *pshared) {
+extern RpcResult_t RPC_MutexAttrGetPshared(RpcMutexAttribute_t *mutexAttr, int *pshared) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_getpshared((pthread_mutexattr_t*)mutexAttr, pshared)) != 0) {
@@ -458,7 +469,7 @@ RpcResult_t RPC_MutexAttrGetPshared(RpcMutexAttribute_t *mutexAttr, int *pshared
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_MutexAttrSetPshared(RpcMutexAttribute_t *mutexAttr, int pshared) {
+extern RpcResult_t RPC_MutexAttrSetPshared(RpcMutexAttribute_t *mutexAttr, int pshared) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_mutexattr_setpshared((pthread_mutexattr_t*)mutexAttr, pshared)) != 0) {
@@ -470,7 +481,7 @@ RpcResult_t RPC_MutexAttrSetPshared(RpcMutexAttribute_t *mutexAttr, int pshared)
 
 /*--- Condition Variable APIs ---*/
 
-RpcResult_t RPC_InitCond(RpcCondID_t *condID, RpcCondAttribute_t *condAttr) {
+extern RpcResult_t RPC_InitCond(RpcCondID_t *condID, RpcCondAttribute_t *condAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_init((pthread_cond_t*)condID, (pthread_condattr_t*)condAttr)) != 0) {
@@ -480,7 +491,7 @@ RpcResult_t RPC_InitCond(RpcCondID_t *condID, RpcCondAttribute_t *condAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondSignal(RpcCondID_t *condID) {
+extern RpcResult_t RPC_CondSignal(RpcCondID_t *condID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_signal((pthread_cond_t*)condID)) != 0) {
@@ -490,7 +501,7 @@ RpcResult_t RPC_CondSignal(RpcCondID_t *condID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondBroadcast(RpcCondID_t *condID) {
+extern RpcResult_t RPC_CondBroadcast(RpcCondID_t *condID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_broadcast((pthread_cond_t*)condID)) != 0) {
@@ -500,7 +511,7 @@ RpcResult_t RPC_CondBroadcast(RpcCondID_t *condID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondWait(RpcCondID_t *condID, RpcMutexID_t *mutexID) {
+extern RpcResult_t RPC_CondWait(RpcCondID_t *condID, RpcMutexID_t *mutexID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_wait((pthread_cond_t*)condID, (pthread_mutex_t*)mutexID)) != 0) {
@@ -510,7 +521,7 @@ RpcResult_t RPC_CondWait(RpcCondID_t *condID, RpcMutexID_t *mutexID) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondWaitTimeout(RpcCondID_t *condID, RpcMutexID_t *mutexID, struct timespec *abstime) {
+extern RpcResult_t RPC_CondWaitTimeout(RpcCondID_t *condID, RpcMutexID_t *mutexID, struct timespec *abstime) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_timedwait((pthread_cond_t*)condID, (pthread_mutex_t*)mutexID, abstime)) != 0) {
@@ -520,7 +531,7 @@ RpcResult_t RPC_CondWaitTimeout(RpcCondID_t *condID, RpcMutexID_t *mutexID, stru
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondDestroy(RpcCondID_t *condID) {
+extern RpcResult_t RPC_CondDestroy(RpcCondID_t *condID) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_cond_destroy(condID)) != 0) {
@@ -532,7 +543,7 @@ RpcResult_t RPC_CondDestroy(RpcCondID_t *condID) {
 
 /*--- Condition Variable Attribute APIs ---*/
 
-RpcResult_t RPC_CondAttrInit(RpcCondAttribute_t *condAttr) {
+extern RpcResult_t RPC_CondAttrInit(RpcCondAttribute_t *condAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_condattr_init((pthread_condattr_t*)condAttr)) != 0) {
@@ -542,7 +553,7 @@ RpcResult_t RPC_CondAttrInit(RpcCondAttribute_t *condAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondAttrDestroy(RpcCondAttribute_t *condAttr) {
+extern RpcResult_t RPC_CondAttrDestroy(RpcCondAttribute_t *condAttr) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_condattr_destroy(condAttr)) != 0) {
@@ -552,7 +563,7 @@ RpcResult_t RPC_CondAttrDestroy(RpcCondAttribute_t *condAttr) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondAttrGetPshared(RpcCondAttribute_t *condAttr, int *pshared) {
+extern RpcResult_t RPC_CondAttrGetPshared(RpcCondAttribute_t *condAttr, int *pshared) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_condattr_getpshared((pthread_condattr_t*)condAttr, pshared)) != 0) {
@@ -563,7 +574,7 @@ RpcResult_t RPC_CondAttrGetPshared(RpcCondAttribute_t *condAttr, int *pshared) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_CondAttrSetPshared(RpcCondAttribute_t *condAttr, int pshared) {
+extern RpcResult_t RPC_CondAttrSetPshared(RpcCondAttribute_t *condAttr, int pshared) {
 	RpcResult_t ret;int err;
 
 	if((err = pthread_condattr_setpshared((pthread_condattr_t*)condAttr, pshared)) != 0) {
@@ -575,7 +586,7 @@ RpcResult_t RPC_CondAttrSetPshared(RpcCondAttribute_t *condAttr, int pshared) {
 
 /*--- Semaphore APIs ---*/
 
-RpcResult_t RPC_SemInit(RpcSem_t *sem, int pshared, int value) {
+extern RpcResult_t RPC_SemInit(RpcSem_t *sem, int pshared, int value) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_init(sem,pshared,value)) != 0) {
@@ -586,7 +597,7 @@ RpcResult_t RPC_SemInit(RpcSem_t *sem, int pshared, int value) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemPost(RpcSem_t *sem) {
+extern RpcResult_t RPC_SemPost(RpcSem_t *sem) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_post(sem)) != 0) {
@@ -597,7 +608,7 @@ RpcResult_t RPC_SemPost(RpcSem_t *sem) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemWait(RpcSem_t *sem) {
+extern RpcResult_t RPC_SemWait(RpcSem_t *sem) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_wait(sem)) != 0) {
@@ -608,7 +619,7 @@ RpcResult_t RPC_SemWait(RpcSem_t *sem) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemWaitTry(RpcSem_t *sem) {
+extern RpcResult_t RPC_SemWaitTry(RpcSem_t *sem) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_trywait(sem)) != 0) {
@@ -623,7 +634,7 @@ RpcResult_t RPC_SemWaitTry(RpcSem_t *sem) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemTimedWait(RpcSem_t *sem,struct timespec *absTime) {
+extern RpcResult_t RPC_SemTimedWait(RpcSem_t *sem,struct timespec *absTime) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_timedwait(sem,absTime)) != 0) {
@@ -634,7 +645,7 @@ RpcResult_t RPC_SemTimedWait(RpcSem_t *sem,struct timespec *absTime) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemGetValue(RpcSem_t *sem,int *value) {
+extern RpcResult_t RPC_SemGetValue(RpcSem_t *sem,int *value) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_getvalue(sem,value)) != 0) {
@@ -645,7 +656,7 @@ RpcResult_t RPC_SemGetValue(RpcSem_t *sem,int *value) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SemDestroy(RpcSem_t *sem) {
+extern RpcResult_t RPC_SemDestroy(RpcSem_t *sem) {
 	RpcResult_t ret;int err;int errNo;
 
 	if((err = sem_destroy(sem)) != 0) {
@@ -658,7 +669,7 @@ RpcResult_t RPC_SemDestroy(RpcSem_t *sem) {
 
 /*--- Shared Memory APIs ---*/
 
-RpcResult_t RPC_ShmOpen(char *name, int oflag, mode_t mode, int *fd, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmOpen(char *name, int oflag, mode_t mode, int *fd, uint32_t *errCode) {
 //	(void)RPC_LogPrint(RPC_LOG_MODULE_SYSCALL,RPC_LOG_LEVEL_DEBUG1,"NOW EXEC RPC_ShmOpen name:%s; oflag:0x%x; mode:O%o fd:0x%x",name, oflag, mode,fd);
 	if((*fd = shm_open(name, oflag, mode)) < 0) {
 		RPC_convertErrno(errno, errCode);
@@ -669,7 +680,7 @@ RpcResult_t RPC_ShmOpen(char *name, int oflag, mode_t mode, int *fd, uint32_t *e
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmUnlink(char *name, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmUnlink(char *name, uint32_t *errCode) {
 	if(shm_unlink(name) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -678,7 +689,7 @@ RpcResult_t RPC_ShmUnlink(char *name, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmClose(int fd, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmClose(int fd, uint32_t *errCode) {
 	if(close(fd) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -687,7 +698,7 @@ RpcResult_t RPC_ShmClose(int fd, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmMap(void *addr, int length, int prot, int flag, int fd, off_t offset, void **allocAddr, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmMap(void *addr, int length, int prot, int flag, int fd, off_t offset, void **allocAddr, uint32_t *errCode) {
 	if((*allocAddr = mmap(addr, length, prot, flag, fd, offset)) == MAP_FAILED) {
 		RPC_convertErrno(errno,errCode);
 		return RPC_SYSCALL_ERROR;
@@ -696,7 +707,7 @@ RpcResult_t RPC_ShmMap(void *addr, int length, int prot, int flag, int fd, off_t
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmUnmap(void *addr,int length, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmUnmap(void *addr,int length, uint32_t *errCode) {
 	if(munmap(addr, length) != 0) {
 		RPC_convertErrno(errno,errCode);
 		return RPC_SYSCALL_ERROR;
@@ -705,7 +716,7 @@ RpcResult_t RPC_ShmUnmap(void *addr,int length, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmFtruncate(int fd, off_t length, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmFtruncate(int fd, off_t length, uint32_t *errCode) {
 	if(ftruncate(fd, length) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -714,7 +725,7 @@ RpcResult_t RPC_ShmFtruncate(int fd, off_t length, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmFstat(int fd, struct stat *buff, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmFstat(int fd, struct stat *buff, uint32_t *errCode) {
 	if(fstat(fd, buff) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -723,7 +734,7 @@ RpcResult_t RPC_ShmFstat(int fd, struct stat *buff, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmFchown(int fd, uid_t owner, gid_t group, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmFchown(int fd, uid_t owner, gid_t group, uint32_t *errCode) {
 	if(fchown(fd, owner, group) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -732,7 +743,7 @@ RpcResult_t RPC_ShmFchown(int fd, uid_t owner, gid_t group, uint32_t *errCode) {
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_ShmFchmod(int fd, mode_t mode, uint32_t *errCode) {
+extern RpcResult_t RPC_ShmFchmod(int fd, mode_t mode, uint32_t *errCode) {
 	if(fchmod(fd, mode) != 0) {
 		RPC_convertErrno(errno, errCode);
 		return RPC_SYSCALL_ERROR;
@@ -743,7 +754,7 @@ RpcResult_t RPC_ShmFchmod(int fd, mode_t mode, uint32_t *errCode) {
 
 /*--- Memory Allocation APIs ---*/
 
-void *RPC_AllocMem(uint32_t size) {
+extern void *RPC_AllocMem(uint32_t size) {
 	void *memP;
 
 	if((memP = malloc(size)) == (void*)NULL) {
@@ -752,7 +763,7 @@ void *RPC_AllocMem(uint32_t size) {
 	return memP;
 }
 
-void *RPC_ReallocMem(void *memP, uint32_t newSize) {
+extern void *RPC_ReallocMem(void *memP, uint32_t newSize) {
 	void *newMemP;
 
 	if((newMemP = realloc(memP, newSize)) == (void*)NULL) {
@@ -761,13 +772,13 @@ void *RPC_ReallocMem(void *memP, uint32_t newSize) {
 	return newMemP;
 }
 
-void RPC_FreeMem(void *memP) {
+extern void RPC_FreeMem(void *memP) {
 	free(memP);
 }
 
 /*--- Time APIs ---*/
 
-RpcResult_t RPC_GetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone, uint32_t *errCode) {
+extern RpcResult_t RPC_GetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone, uint32_t *errCode) {
 	if(gettimeofday(timeVal,timeZone) != 0) {
 		RPC_convertErrno(errno,errCode);
 		return RPC_SYSCALL_ERROR;
@@ -776,7 +787,7 @@ RpcResult_t RPC_GetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone,
 	return RPC_SUCCESS;
 }
 
-RpcResult_t RPC_SetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone, uint32_t *errCode) {
+extern RpcResult_t RPC_SetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone, uint32_t *errCode) {
 	if(settimeofday(timeVal,timeZone) != 0) {
 		RPC_convertErrno(errno,errCode);
 		return RPC_SYSCALL_ERROR;
@@ -787,11 +798,11 @@ RpcResult_t RPC_SetTimeOfDay(struct timeval *timeVal, struct timezone *timeZone,
 
 /*--- Process APIs ---*/
 
-uint32_t RPC_GetPid(void) {
+extern uint32_t RPC_GetPid(void) {
 	return (uint32_t)getpid();
 }
 
-RpcResult_t RPC_Fork(int32_t *pid,uint32_t *errCode) {
+extern RpcResult_t RPC_Fork(int32_t *pid,uint32_t *errCode) {
 	if((*pid = (int32_t)fork()) == 0) {
 		*errCode = RPC_NO_ERROR;
 		return RPC_SUCCESS;
@@ -806,7 +817,7 @@ RpcResult_t RPC_Fork(int32_t *pid,uint32_t *errCode) {
 	}
 }
 
-RpcResult_t RPC_Exec(char *filePath,char *argv[],char *envp[],uint32_t *errCode) {
+extern RpcResult_t RPC_Exec(char *filePath,char *argv[],char *envp[],uint32_t *errCode) {
 	if(execve(filePath,argv,envp) == -1) {
 		RPC_convertErrno(errno,errCode);
 		return RPC_SYSCALL_ERROR;
@@ -850,17 +861,17 @@ extern RpcResult_t RPC_Kill(int32_t pid,int32_t sig,uint32_t *errCode) {
 
 /*--- Other APIs ---*/
 
-void RPC_Srand(uint32_t seed) {
+extern void RPC_Srand(uint32_t seed) {
 	static RpcBool_t srandCallFlag = RPC_FALSE;
 
 	if(!srandCallFlag) {
 		srandCallFlag = RPC_TRUE;
-		srand(seed);
+		srandom(seed);
 	}
 }
 
-uint32_t RPC_Rand(void) {
-	return rand();
+extern uint32_t RPC_Rand(void) {
+	return random();
 }
 
 
