@@ -50,8 +50,8 @@
 
 #include	"ipcTest.h"
 
-//#define	MULTICAST_TEST
-#undef	MULTICAST_TEST
+#define	MULTICAST_TEST
+//#undef	MULTICAST_TEST
 
 RpcResult_t queTest(RpcBool_t globalFlag);
 RpcResult_t shmTest(void);
